/*Explicit Model: Using Functions (Macros)*/

c_decl { #include <math.h> } 
#include "newExMo.h" 
#include "newExMoInLines.txt" 	
#define OBMAX 4
#define MAX_OMEGAD 1

/*Define a polar coordinate to be a distance and angle from origin (pole)*/
/*(Pole: centre of the environment. Polar axis: vertical line directed north.)*/
typedef polarCoord {int d; int a}; 

/*Setting the C_Track variables*/
c_track"&x" "sizeof(double)"
c_track "&prevX" "sizeof(double)"

c_track "&roboA" "sizeof(double)"
c_track "&enviD" "sizeof(long double)"
c_track "&enviA" "sizeof(long double)"

c_track "&obD" "sizeof(double)"
c_track "&obA" "sizeof(double)"
c_track "&relD" "sizeof(double)"
c_track "&relA" "sizeof(double)"

c_track "&obDist" "sizeof(int)"
c_track "&obAng" "sizeof(int)"
c_track "&moveDist" "sizeof(int)"
c_track "&inCone" "sizeof(int)"

/*Array of obstacles in the fixed environment*/
polarCoord arrObs[OBMAX]; 

/*Robot is initiated in the centre of the environment*/
int roboAng;
int enviDist; 
int enviAng ; 
int omegaD;
int sig; 
int prevSig;
byte doWrap; 
byte headOn;
int relDist;
	
proctype robot() {	
	do
	:: (doWrap==0) ->		d_step{	SCAN_APPROACHING_OBS(); 
				RESPOND(); 
				MOVE_ROBOT();
				HEAD_ON();
			}; 
	:: (doWrap==1) ->		d_step{	WRAP();
			}; 
	od;
};


init {
	d_step{ 
		/* Set up the polar coordinates of the obstacles - fixed for model */
		/*(This setup will be automated with a C function.)*/


                SETENV6();



		roboAng = 0;
		enviDist = 0;
		enviAng = 0; 
		omegaD = 0; 
		doWrap = 0; 
		sig = 0;
		prevSig = 0;
		headOn = 0;
		/*Relative distance should be set to outwith the "cone
		of influence" initially to stop property violations.*/
		relDist = 200;

		c_code{ 
			roboA = 0; 
			enviA = 0;
			enviD = 0;
		};
	};
	atomic{ run robot();};
};


#define p ((sig<=-6)&&(sig>=6))
#define d ((prevSig!=0) && (prevSig>-6) && (prevSig<6) ) 
#define a (omegaD <= MAX_OMEGAD)

/*ltl p1 { <>[](p->!d)}*/
ltl p2 { [] a }


